-------------------------------------------------------------------
-- Date 		: 05 Jan 2011
-- Databse		: Ms SQL Server
-- Description 		: ISAMESSO Audit Log Housekeeping Maintenance Script
-- IMS Version		: 8.1.x / 8.2.x
--
--  Syntax : sp_pruneImsLogOlderThan '<del mode>', <days> 
--  * <del mode> - Mandatory, text, case sensitive  
--				 - ''ABOUT'' display script information 
--				 - ''DELETE'' to perform actual delete 
--				 - ''HELP'' display script syntax and example 
--				 - ''SIMULATE'' to simulate deletion without actual delete 
--  * <days> - Mandatory, numeric  
--           - Enter 0 to delete all 
--
-- Example : 
--  * exec sp_pruneImsLogOlderThan 'SIMULATE',30  - Simulate deletion of audit log older than 30 days without performing actual delete
--  * exec sp_pruneImsLogOlderThan 'DELETE',14    - Delete audit log older than 2 weeks (14 days) from database
--  * exec sp_pruneImsLogOlderThan 'DELETE',0     - Delete all audit log from database
--
-------------------------------------------------------------------
--
-- DROP PROCEDURE sp_pruneImsLogOlderThan;
--
-- EXEC AS USER = 'imsdb7907';
-- exec sp_pruneImsLogOlderThan 'ABOUT',30;
--
-- Tables affected : 
--  IMSLOGUserActivity 
--  IMSLOGUserAdminActivity 
--	IMSLOGUserService
--	IMSLOGSystemMgmtActivity
--	IMSLOGSystemOps(if exist)
--
-- Changes Log :
-- v1.0.1 - Fix HELP information example 
--        - Verify ISAMESSO schema even on 'ABOUT' and 'HELP' mode
--		  - Added ISAMESSO schema name in the About
--

CREATE PROCEDURE sp_pruneImsLogOlderThan
@IN_operationCmd VARCHAR(8),
@IN_days INT
AS
BEGIN

	DECLARE @var_strSQL NVARCHAR(128) ;
	DECLARE @var_bitLoop BIT;
		
	DECLARE @var_tsRetainDate DATETIME;
	DECLARE @var_iBatchNum INT ; SET @var_iBatchNum = 10000;
	DECLARE @var_iTmpNum INT ; 
	
	DECLARE @var_info_script_ver VARCHAR(16); SET @var_info_script_ver = '1.0.1';
	DECLARE @var_info_db_ver VARCHAR(128);
	
	DECLARE @var_info_start_time DATETIME;
	DECLARE @var_info_end_time DATETIME;
	DECLARE @var_info_elapsed_time VARCHAR(32);
	
	DECLARE @var_ims_schema VARCHAR(32); 
	DECLARE @var_tableschema VARCHAR(32); 

	DECLARE @var_showSyntax BIT; SET @var_showSyntax = 0;
	DECLARE @var_invalid_schema BIT; SET @var_invalid_schema = 0;
	DECLARE @var_del_result INT ; SET @var_del_result = 0;
	
	SET @var_tsRetainDate = DATEADD(day,-@IN_days,CURRENT_TIMESTAMP);

		
	--IF LEN(RTRIM(@IN_imsSchema))>0 AND @IN_imsSchema IS NOT NULL
	--	SET @var_ims_schema = @IN_imsSchema
	--ELSE
		SET @var_ims_schema = SCHEMA_NAME();
		
	SET @var_tableschema = @var_ims_schema + '.';
	
	-- verify IMS schema
	DECLARE @var_iLogTableCount INT;
	SET @var_iLogTableCount = (SELECT COUNT(*) FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = @var_ims_schema AND UPPER(TABLE_NAME) like 'IMSLOG%');
	IF(@var_iLogTableCount<4)
		SET @var_invalid_schema = 1;

	IF @var_invalid_schema = 0
	BEGIN

		PRINT N'command : sp_pruneImsLogOlderThan ''' + @IN_operationCmd + ''', ' + CAST(@IN_days AS VARCHAR);

		IF @IN_operationCmd IN ('DELETE','SIMULATE')
		BEGIN
			PRINT N' Input  > Delete mode : ' + @IN_operationCmd;
			PRINT N' Input  > Days : ' + CAST(@IN_days AS VARCHAR);
			PRINT N' Output > Prune records older than (YYYY.MM.DD) : ' + CONVERT(NVARCHAR, @var_tsRetainDate, 102);
			PRINT N' ';
			
			SET NOCOUNT ON;
  			-- Capture start time
			SET @var_info_start_time = CURRENT_TIMESTAMP;

			DECLARE @var_strAction VARCHAR(16);
			IF @IN_operationCmd = 'DELETE'
				SET @var_strAction = 'deleted';
			ELSE
				SET @var_strAction = 'calculating';	
			
			----------------------
			-- IMSLOGUserActivity
			----------------------
			SET @var_del_result = 0;
			IF @IN_operationCmd = 'DELETE'
			BEGIN
				SET @var_bitLoop = 1;
				SET @var_strSQL = N'DELETE TOP('+CAST(@var_iBatchNum as NVARCHAR)+')FROM '+@var_tableschema+'IMSLOGUserActivity WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				WHILE @var_bitLoop=1
				BEGIN
					BEGIN TRAN
						EXECUTE(@var_strSQL);
						SET @var_iTmpNum = @@ROWCOUNT;
						SET @var_del_result = @var_del_result + @var_iTmpNum;
						IF (@var_iTmpNum < @var_iBatchNum) SET @var_bitLoop = 0;
					COMMIT TRAN;
				END;
			END ELSE
			BEGIN
				SET @var_strSQL = N'SELECT @result = COUNT(*) FROM '+@var_tableschema+'IMSLOGUserActivity WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				EXECUTE sp_executesql @var_strSQL, N'@result int OUTPUT', @result=@var_del_result OUTPUT;
			END;
			PRINT N' - '+@var_strAction+' IMSLOGUserActivity ... '+CAST(@var_del_result AS VARCHAR);
			----------------------
			-- IMSLOGUserAdminActivity
			----------------------
			SET @var_del_result = 0;
			IF @IN_operationCmd = 'DELETE'
			BEGIN
				SET @var_bitLoop = 1;
				SET @var_strSQL = N'DELETE TOP('+CAST(@var_iBatchNum as NVARCHAR)+')FROM '+@var_tableschema+'IMSLOGUserAdminActivity WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				WHILE @var_bitLoop=1
				BEGIN
					BEGIN TRAN
						EXECUTE(@var_strSQL);
						SET @var_iTmpNum = @@ROWCOUNT;
						SET @var_del_result = @var_del_result + @var_iTmpNum;
						IF (@var_iTmpNum < @var_iBatchNum)  SET @var_bitLoop = 0;
					COMMIT TRAN;
				END;
			END ELSE
			BEGIN
				SET @var_strSQL = N'SELECT @result = COUNT(*) FROM '+@var_tableschema+'IMSLOGUserAdminActivity WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				EXECUTE sp_executesql @var_strSQL, N'@result int OUTPUT', @result=@var_del_result OUTPUT;
			END;
			PRINT N' - '+@var_strAction+' IMSLOGUserAdminActivity ... '+CAST(@var_del_result AS VARCHAR);
			----------------------
			-- IMSLOGUserService
			----------------------
			SET @var_del_result = 0;
			IF @IN_operationCmd = 'DELETE'
			BEGIN
				SET @var_bitLoop = 1;
				SET @var_strSQL = N'DELETE TOP('+CAST(@var_iBatchNum as NVARCHAR)+')FROM '+@var_tableschema+'IMSLOGUserService WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				WHILE @var_bitLoop=1
				BEGIN
					BEGIN TRAN
						EXECUTE(@var_strSQL);
						SET @var_iTmpNum = @@ROWCOUNT;
						SET @var_del_result = @var_del_result + @var_iTmpNum;
						IF (@var_iTmpNum < @var_iBatchNum) SET @var_bitLoop = 0;
					COMMIT TRAN;
				END;
			END ELSE
			BEGIN
				SET @var_strSQL = N'SELECT @result = COUNT(*) FROM '+@var_tableschema+'IMSLOGUserService WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				EXECUTE sp_executesql @var_strSQL, N'@result int OUTPUT', @result=@var_del_result OUTPUT;
			END;
			PRINT N' - '+@var_strAction+' IMSLOGUserService ... '+CAST(@var_del_result AS VARCHAR);
			----------------------
			-- IMSLOGSystemMgmtActivity
			----------------------
			SET @var_del_result = 0;
			IF @IN_operationCmd = 'DELETE'
			BEGIN
				SET @var_bitLoop = 1;
				SET @var_strSQL = N'DELETE TOP('+CAST(@var_iBatchNum as NVARCHAR)+')FROM '+@var_tableschema+'IMSLOGSystemMgmtActivity WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				WHILE @var_bitLoop=1
				BEGIN
					BEGIN TRAN
						EXECUTE(@var_strSQL);
						SET @var_iTmpNum = @@ROWCOUNT;
						SET @var_del_result = @var_del_result + @var_iTmpNum;
						IF (@var_iTmpNum < @var_iBatchNum) SET @var_bitLoop = 0;
					COMMIT TRAN;
				END;
			END ELSE
			BEGIN
				SET @var_strSQL = N'SELECT @result = COUNT(*) FROM '+@var_tableschema+'IMSLOGSystemMgmtActivity WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
				EXECUTE sp_executesql @var_strSQL, N'@result int OUTPUT', @result=@var_del_result OUTPUT;
			END;
			PRINT N' - '+@var_strAction+' IMSLOGSystemMgmtActivity ... '+CAST(@var_del_result AS VARCHAR);

			----------------------
			-- IMSLOGSystemOps
			----------------------
			SET @var_del_result = 0;
			DECLARE @var_iLogSystemOpsExist INT;
			SET @var_iLogSystemOpsExist = (SELECT COUNT(*) FROM INFORMATION_SCHEMA.TABLES WHERE UPPER(TABLE_NAME) = 'IMSLOGSYSTEMOPS');
			-- Check if IMSLogSystemOps exist
			IF(@var_iLogSystemOpsExist>0)
			BEGIN
				IF @IN_operationCmd = 'DELETE'
				BEGIN
					SET @var_bitLoop = 1;
					SET @var_strSQL = N'DELETE TOP('+CAST(@var_iBatchNum as NVARCHAR)+')FROM '+@var_tableschema+'IMSLOGSystemOps WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
					WHILE @var_bitLoop=1
					BEGIN
						BEGIN TRAN
							EXECUTE(@var_strSQL);
							SET @var_iTmpNum = @@ROWCOUNT;
							SET @var_del_result = @var_del_result + @var_iTmpNum;
							IF (@var_iTmpNum < @var_iBatchNum) SET @var_bitLoop = 0;
						COMMIT TRAN;
					END;
				END ELSE
				BEGIN
					SET @var_strSQL = N'SELECT @result = COUNT(*) FROM '+@var_tableschema+'IMSLOGSystemOps WHERE logTime < '''+CAST(@var_tsRetainDate as varchar)+'''';
					EXECUTE sp_executesql @var_strSQL, N'@result int OUTPUT', @result=@var_del_result OUTPUT;
				END;
				PRINT N' - '+@var_strAction+' IMSLOGSystemOps ... '+CAST(@var_del_result AS VARCHAR);
			END;

			-- Capture end time
			SET @var_info_end_time = CURRENT_TIMESTAMP;
			DECLARE @var_timediff DATETIME; SET @var_timediff = @var_info_end_time-@var_info_start_time;
			SET @var_info_elapsed_time = CAST(DATEPART(day,@var_timediff)-1 AS VARCHAR)+'d ' 
				+ CAST(DATEPART(hour,@var_timediff) AS VARCHAR) +'h ' + CAST(DATEPART(minute,@var_timediff) AS VARCHAR)+'m '
				+ CAST(DATEPART(second,@var_timediff) AS VARCHAR)+'s ' + CAST(DATEPART(millisecond,@var_timediff) AS VARCHAR)+'ms ';

			PRINT N'';
			PRINT N' Start time   : ' + CONVERT(NVARCHAR, @var_info_start_time, 121);
			PRINT N' End time     : ' + CONVERT(NVARCHAR, @var_info_end_time, 121);
			PRINT N' Elapsed time : ' + @var_info_elapsed_time;
			SET NOCOUNT OFF;
		END	
		ELSE IF @IN_operationCmd = 'ABOUT'
		BEGIN
			SET @var_info_db_ver = REPLACE(REPLACE(@@VERSION,CHAR(10),' '),'	','');
			PRINT N' ISAMESSO Audit Log Housekeeping Maintenance Script';
			PRINT N'  - Database Name 	 : ' + DB_NAME();
			PRINT N'  - Schema Name 	 : ' + @var_ims_schema;
			PRINT N'  - Script Version   : ' + @var_info_script_ver;
			PRINT N'  - Database Version : ' + @var_info_db_ver;

		END ELSE 
		BEGIN	
			IF @IN_operationCmd <> 'HELP'
			BEGIN
				PRINT N' Error : Invalid delete mode '''+@IN_operationCmd+'''';
				PRINT N' ';
			END;
			SET @var_showSyntax = 1;
		END;
	END ELSE
	BEGIN
		PRINT N' Error : Invalid IMS schema '''+@var_ims_schema+'''';
		PRINT N' ';
		SET @var_showSyntax = 1;
	END;

	IF @var_showSyntax = 1
	BEGIN
		PRINT N' Syntax : sp_pruneImsLogOlderThan ''<del mode>'', <days> ';
		PRINT N' * <del mode> - Mandatory, text, case sensitive';
		PRINT N'              - ''ABOUT'' display script information';
		PRINT N'              - ''DELETE'' to perform actual delete ';
		PRINT N'              - ''HELP'' display script syntax and example';
		PRINT N'              - ''SIMULATE'' to simulate deletion without actual delete';
		PRINT N' * <days> - Mandatory, numeric';
		PRINT N'          - Enter 0 to delete all';
		PRINT N'';
		PRINT N'Example : ';
		PRINT N' * sp_pruneImsLogOlderThan ''SIMULATE'',30 - Simulate deletion of audit log older than 30 days without performing actual delete'
		PRINT N' * sp_pruneImsLogOlderThan ''DELETE'',14   - Delete audit log older than 2 weeks (14 days) from database'
		PRINT N' * sp_pruneImsLogOlderThan ''DELETE'',0    - Delete all audit log from database'
		
	END;
	
END
